% OBJective function for GReenHouSe/gemuese model 10
%
% This function implements the greenhouse/Gemuese model.
%
% Syntax:  [ObjVal, t, x] = objgrhs10(Chrom, Method, TSTART, TEND, DataKlimaPlant, XINIT, PRICES)
%
% Input parameters:
%    Chrom     - Matrix containing the chromosomes of the current
%                population. Each row corresponds to one individual's
%                string representation.
%                if Chrom == [], then special values will be returned
%                format:   1. ControlVariable  2. ControlVariable ...  NCONTR. ControlVariable
%                    Ind1  1:ControlSteps      1:ControlSteps     ...  1:ControlSteps
%                    Ind2  1:ControlSteps      1:ControlSteps     ...  1:ControlSteps
%    Method    - if Chrom == [] and
%                Method == 1 (or []) return boundaries
%                Method == 2 return title
%                Method == 3 return value of global minimum
%                if Chrom ~[], plant model for simulation
%    TSTART    - (optional) start time
%    TEND      - (optional) end time
%    DataKlimaPlant - (optional) Matrix containing data of Klima and Plant 
%                    for the time period defined in TSTART and TEND
%                if empty, klidata4 will be called to get the data
%    XINIT     - (optional) initial values of state variables
%                if empty, startdat will be called to get the data
%    PRICES    - (optional) prices of control and gemuese
%
% Output parameters:
%    ObjVal    - Column vector containing the objective values of the
%                individuals in the current population.
%                if called with Chrom == [], then ObjVal contains
%                Method == 1, matrix with the boundaries of the function
%                Method == 2, text for the title of the graphic output
%                Method == 3, value of global minimum
%    t         - vector of time (steps)
%    x         - matrix containing state values at time steps
%
% See also: geamain, plotgrh3, simgrhs4, recvgrhs, klidata4

% Author:   Hartmut Pohlheim
% History:  13.03.97    file created
%           25.06.98    adaptation fpr Matlab5


function [ObjVal, t, x] = objgrhs10(Chrom, Method, TSTART, TEND, DataKlimaPlant, XINIT, PRICES);

% Compute population parameters
   [Nind, Nvar] = size(Chrom);

% Parameters of objective function
   NCONTR = 3;          % number of control variables (every time step)
   StepHour = 30;       % number of steps per hour
   StepControlHour = 4;  % number of control values per hour (smaller than StepHour)

% end conditions
%          DDI TEMPI CI BIOM GEWINN FI
   XEND = [  0;  0;   0;   0;   0;   0];
   XMIN = [  0; 16;   0;-1e5;-1e5; -10];
   XMAX = [ 90; 35; 2e3; 1e5; 1e5; 100];

% weights for min., max. and end of states and control values
   XENDWEIGHT = 0 * [ 1;    1; 1; 1; 1; 1];       % XEND
   XMINWEIGHT =     [ 1; 1e-5; 1; 1; 1; 1];       % XMIN
   XMAXWEIGHT =     [ 1; 1e-5; 1; 1; 1; 1];       % XMAX
   % UWEIGHT = 0.5 * [ 1; 1; 1];                  % Control
   
% Check size of Chrom and do the appropriate thing
   % if Chrom is [], then
   if Nind == 0
      % Default dimension of objective function
      TDAUER = TEND - TSTART;
      Dim = ceil(StepControlHour * TDAUER / 3600 + 1);
      % return text of title for graphic output
      if Method == 2
         ObjVal = ['Greenhouse climate system 4'];
      % return value of global minimum
      elseif Method == 3
         ObjVal = 0;    % not correct, only for compatiblity
      % define size of boundary-matrix and values
      else   
         % lower and upper bound, identical for all Dim variables,
         % here 3 control variables
         % define boundaries in dependence of TSTART, in winter no ventilation and so on
         % Compute the month from time
         MonthInternal = compgrhs('convert_time2month', TSTART);
         MonthInternal = MonthInternal(1);
         ObjVal = zeros(2,3);
         ObjVal(1,:) = [  0,   1,   0];
         % Month =   [  1   2   3   4   5   6   7   8   9  10  11  12]
         ObjValMax = [200 200 180 150 120  80  80  80 120 150 200 200;     % Heating
                       10  10  50 100 150 200 200 200 100  50  10  10;     % Ventilation
                       10  10  10  10  10  10  10  10  10  10  10  10      % CO2 injection
                     ];
         ObjVal(2,:) = ObjValMax(:, MonthInternal)';
         % expand boundaries
         % format:       1. ControlVariable  2. ControlVariable ...  NCONTR. ControlVariable
         %  lower bound  1:Dim               1:Dim              ...  1:Dim
         %  upper bound  1:Dim               1:Dim              ...  1:Dim
         ObjVal = expandm(ObjVal, [1, Dim]);
      end
   % compute values of function
   else
      % Define used plant model
      if nargin < 2, Method = []; end
      if isempty(Method), MODEL = 3;     % 1 - Tomate
      else MODEL = Method;               % 2 - Paprika I
      end                                % 3 - Paprika II
      SIM_F = 'simsgrhs';

      % Set default values, if not defined
      if nargin < 3, TSTART = []; end; if isempty(TSTART), TSTART = 0; end
      if nargin < 4, TEND = [];   end; if isempty(TEND),   TEND = TSTART + 4 * 3600; end
      if nargin < 5, DataKlimaPlant = [];  end
      if isempty(DataKlimaPlant), DataKlimaPlant = compgrhs('lookup_klimaplant', TSTART, TEND); end

      % initial conditions
      if nargin < 6, XINIT = []; end

      if nargin < 7, PRICES = []; end
 
      % Start computation of objective function

      % If only one control value is defined for the whole simulation
      % period, expandm to two values (needed for interpolation inside sim*.m)
      if Nvar/NCONTR == 1,
         Chrom = expandm(Chrom, [1, 2]);
         Nvar = 2 * Nvar; % ControlSteps = 2;
      end
      TIMEVEC = linspace(TSTART, TEND, Nvar/NCONTR)';

      assignin('base', 'MODEL', MODEL);
      assignin('base', 'DataKlimaPlant', DataKlimaPlant);
      assignin('base', 'PRICES', PRICES);

      % Compute better start values for the states of the simulation
      % by starting a short simulation (4 minutes) and using the returned
      % end states as initial states for the long simulation, this produces
      % more correct results (the high changes at the beginning of a simulation
      % due to incorrect initial state values are removed)
      if isempty(XINIT), 
         xinit1 = startdt3(TSTART, MODEL);
         TENDINI = TSTART+10*60;
         StartChrom = [[TSTART; TENDINI] , repmat(Chrom(1,[1, Nvar/NCONTR+1, 2*Nvar/NCONTR+1]), [2, 1])];
         
         SimOpts = simset('Solver', 'ode2', 'FixedStep', 1/StepHour*3600, 'InitialState', xinit1);
         [t, x] = sim(SIM_F, [TSTART, TENDINI], SimOpts, StartChrom); % , MODEL, DataKlimaPlant, PRICES);
         % [t, x] = intrk4(SIM_F, [TSTART, TSTART+10*60], xinit1, ...
         %                 [1e-4, 1/StepHour*3600, 4/StepHour*3600, NCONTR], ...
         %                 StartChrom, MODEL, DataKlimaPlant, PRICES);

         xinit2 = x(size(x, 1),:);
         % Set xinit for biomass and profit to zero
         XINIT = [xinit2(1:3) 0 0 xinit2(6)];
      else
         XINIT = startdt3(TSTART, MODEL);
      end
      XINIT = repmat(XINIT, [Nind, 1]);

      % Define method of integration
      sim = 1;

      % Use Matlab's ODE-functions with or without step-size control, slow for many individuals
      % however, may be used to look for smallest step size
      if sim == 1,  
         x = []; poses = []; TimeSimall = []; ObjValTempi = []; ObjValGewinn = [];
         tic
         for irun = 1:Nind,
            fprintf('%g ', irun);
            Control = [TIMEVEC, reshape(Chrom(irun, :), Nvar/NCONTR, NCONTR)];
            SimOpts = simset('Solver', 'ode2', 'FixedStep', 1/StepHour*3600, 'InitialState', XINIT(irun,:));
            [t, xres] = sim(SIM_F, [TSTART, TEND], SimOpts, Control); % , MODEL, DataKlimaPlant, PRICES)

            % old for Matlab4
            % [t, xres] = rk45(SIM_F, [TSTART, TEND], XINIT(irun,:), ...
            %                  [1e-5, .4/StepHour*3600, 4/StepHour*3600], ...
            %                  Control, MODEL, DataKlimaPlant, PRICES);

            x = [x; xres];
            poses = [poses; size(xres, 1)]; TimeSimall = [TimeSimall; t];

            % Calculate objective function
            % constraints
            ValConMin = xres - repmat(XMIN', [size(t, 1), 1]);
            ValConMin = ValConMin .* (ValConMin < 0);
            ObjValMin(irun, :) = XMINWEIGHT' .* trapz(t, ValConMin.^2);
            ValConMax = xres - repmat(XMAX', [size(t, 1), 1]);
            ValConMax = ValConMax .* (ValConMax > 0);
            ObjValMax(irun, :) = XMAXWEIGHT' .* trapz(t, ValConMax.^2);

            % old variante of above
            % ValTempi = xres(:,2) - XMIN(2), ValTempi = ValTempi .* (ValTempi < 0)
            % ObjValTempi(irun, :) = XMINWEIGHT(2) * trapz(t, ValTempi.^2)
         end
         toc
         % [ t./3600, [0; diff(t./3600)], x([1:poses(1)],1:5)]

         % Part of objective value for violation of constraints
         ObjValCon = sum(ObjValMin')' + sum(ObjValMax')';

         % Standard part of objective values (without constraints, 
         %    only GEWINN at end of simulation)
         ObjValGewinn = -x(cumsum(poses), 5);

      elseif sim == 2,  % use vectorized-runge-kutta 4, quick for many individuals
         [t, x] = intrk4(SIM_F, [TSTART, TEND], XINIT, ...
                          [1e-4, 1/StepHour*3600, 4/StepHour*3600, NCONTR], ...
                          Chrom, MODEL, DataKlimaPlant, PRICES);
         % poses = repmat(size(t, 1), [Nind, 1]); TimeSimall = expandm(t, [Nind, 1]);
         % [t./3600, [0; diff(t./3600)], x(1:Nind:Nind*(size(t, 1)-1)+1,1:5)]
         
         % Calculate objective function

         % Look for violation of constraints, weight the violation and add all together
         ValConMin = x - repmat(XMIN', [size(t, 1)*Nind, 1]);
         ValConMin = ValConMin .* (ValConMin < 0);
         ValConMinTempi = reshape(ValConMin(:,2), Nind, size(t, 1));
         ValConMinTempi =  repmat(XMINWEIGHT(2)', [Nind, 1]) .* trapz(t, (ValConMinTempi.^2)')';
         ValConMax = x - repmat(XMAX', [size(t, 1)*Nind, 1]);
         ValConMax = ValConMax .* (ValConMax > 0);
         ValConMaxTempi = reshape(ValConMax(:,2), Nind, size(t, 1));
         ValConMaxTempi =  repmat(XMAXWEIGHT(2)', [Nind, 1]) .* trapz(t, (ValConMaxTempi.^2)')';
         %   ValTempi = []; for irun = 1:size(t, 1), ValTempi = [ValTempi, x(((irun-1)*Nind)+1:irun*Nind, 2)]; end

         % Part of objective value for violation of constraints
         ObjValCon = ValConMinTempi + ValConMaxTempi;

         % Standard part of objective values (without constraints, 
         %    only GEWINN at end of simulation)
         ObjValGewinn = -x(size(x,1)-Nind+1:size(x,1),5);

      end

      % Addition of all objective values (GEWINN and constraints)
      ObjVal = ObjValGewinn + ObjValCon;
      
      % To ensure that Feuchte (state 6) is always >=0
      x(:,6) = x(:,6) .* (x(:,6) > 0);

   end   


% End of function

