% REsult ConVersion of GReenHouSe/gemuese simulation 3
%
% Umwandlung der Optimierungsergebnisse der Gewaechshaus/Gemuesesimulation
% zur Abspeicherung und Dokumentation
%
% This function takes the best individual, simulates it using the objective
% function and converts the results (time and state values) into
% a readable format for saving and documentation.
%
% Syntax:  ObjVal = recvgrh3(xnew, MODEL, TSTART, TEND, MONTH, XINIT, PRICES, FileSave, PrintStep, PrintHeader)
%
% Input parameters:
%    OBJ_F     - String containing the name of the objective function
%    xnew      - Vector (matrix) containing the best chromosomes of the current
%                   population. Each row corresponds to one individual's
%                   string representation.
%    MODEL     - Scalar containing method of simulation
%    TSTART    - (optional) start time in seconds
%    TEND      - (optional) end time in seconds
%    DataKlimaPlant - (optional) Matrix containing data of Klima and Plant
%    XINIT     - (optional) initial values of state variables
%    PRICES    - (optional) prices for control and gemuese
%    FileSave  - (optional) name of file for saving results
%                   if empty or omitted: 'grh3_001.sim' is assumed
%    PrintStep - (optional) time interval, at which to print result values,
%                   if empty or omitted PrintStep = 1/6 (10 minutes) is assumed
%    PrintHeader-(optional) defines, if header should be printed,
%                    0: no print, else: print
%
% Output parameters:
%    startdatnext - Row vector containing the state values of TEND
%                could be used for initializing the next optimization
%
% See also: objgrhs, plotgrhs

% Author:   Hartmut Pohlheim
% History:  11.03.97    file created
%                       new parameter OBJ_F added for more flexibility


function [startdatnext] = recvgrh3(OBJ_F, ControlVars, MODEL, TSTART, TEND, DataKlimaPlant, XINIT, PRICES, FileSave, PrintStep, PrintHeader);   

% Check input parameters
   if nargin < 11, PrintHeader = []; end
   if isempty(PrintHeader), PrintHeader = 1;
   elseif (PrintHeader ~= 0), PrintHeader = 1;
   end

   if nargin < 10, PrintStep = []; end
   if isempty(PrintStep), PrintStep = 1/6; end

   if nargin < 9, FileSave = []; end
   if isempty(FileSave), FileSave = 'grh3_001.sim'; end

   if nargin < 8, PRICES = []; end
   if nargin < 7, XINIT = []; end
   if nargin < 6, DataKlimaPlant = []; end
   if nargin < 5, error('Not enough input parameters!'); end

   if size(ControlVars, 1) > 1, ControlVars = ControlVars(1,:); end

   % Calculate states from objective function
   [ObjValres, t, States]= feval(OBJ_F, ControlVars, MODEL, TSTART, TEND, DataKlimaPlant, XINIT, PRICES); end

   % Plot the results, use standard problem specific plot function
   % plotgrh3(OBJ_F, ControlVars, 1, MODEL, TSTART, TEND, DataKlimaPlant, XINIT, PRICES);

% Calculate size of variables
   [NCon1, NCon2] = size(ControlVars);
   [NSta1, NSta2] = size(States);
   NCONTR = 3;
   ControlSteps = NCon2/NCONTR;

% Convert result data
   TIMEVEC = linspace(TSTART, TEND, ControlSteps)';
   
   % if NSta1 ~= ControlSteps, 
   ControlTable = reshape(ControlVars, ControlSteps, 3);
   ControlVars = table1([TIMEVEC, ControlTable], t);
   % end   

   % Create matrix with all results
   % time (in hours), all state values (5), Feuchte innen (state(6)), control values (3)
   ResultsAll = [t  States  feuchtei(States(:,1), States(:,2))  ControlVars];

   % Print only the result at specified time points, given by PrintStep
   % ResultsAll = table1(ResultsAll, (TSTART:PrintStep:TEND)');   % best method, but doesn't work, because out of memory
   PrintStepNum = ceil(PrintStep / (ResultsAll(2,1)/3600-ResultsAll(1,1)/3600));
   ResultsAll = ResultsAll(1:PrintStepNum:size(ResultsAll,1),:);
   Time = ResultsAll(:,1);
   ResTime = compgrhs('convert_time2dayhour', Time);

   % Get data for Klima and Plant
   if isempty(DataKlimaPlant),
      [KLIMA, PLANT] = klidata4(Time);
   else
      KlimaPlant = table1(DataKlimaPlant, Time);
      KLIMA = KlimaPlant(:,1:7);
      PLANT = KlimaPlant(:,8:10);
   end
   % Compute temperature of greenhouse cover
   KlimaX2 = ResultsAll(:,3);
   for iklima = 1:length(Time),
      KLIMA(iklima, 2) = calctemg(KlimaX2(iklima), KLIMA(iklima, 3), KLIMA(iklima,6), Time(iklima));
   end
   
   % Open the file for saving
   [fid, error_message] = fopen(FileSave, 'at');
   if fid == -1, error(sprintf('error during fopen: %s', error_message));
   else
      % Write a header with information to the text file
      if PrintHeader == 1,
         datetime = clock; time = sprintf('%.0f:%02.0f:%02.0f', datetime(4), datetime(5), datetime(6));
         fprintf(fid, '\n\n\nDate: %s   Time: %s\n', date, time);
         [Du, TSDayHour] = compgrhs('convert_time2dayhour', TSTART);
         [Du, TSMonth] = compgrhs('convert_time2month', TSTART);
         [Du, TEDayHour] = compgrhs('convert_time2dayhour', TEND);
         [Du, TEMonth] = compgrhs('convert_time2month', TEND);
         fprintf(fid, 'Parameters:  model = %d   TStart: %s Uhr (%s)  TEnd: %s Uhr (%s)   prices = ',...
                      MODEL, TSDayHour, TSMonth, TEDayHour, TEMonth);
         if isempty(PRICES), fprintf(fid, 'default\n'), else fprintf(fid, '%g  ', PRICES); fprintf(fid, '\n'); end
         TitleString = ['    Zeit_d';'    Zeit_s';'  DDI_g/m';'   TEMI_C';'    CI_ppm';' BIOM_g/m';' GEW_DM/m';...
                        ' KOND_g/m';'    FI_%rf';'    Q_W/m';' LR_m/mh';'   W_g/mh';'   TEMB_C';'   TEMG_C';...
                        '   TEMA_C';'    FA_%rF';'    CA_ppm';'IGLOB_W/m';'     U_m/s';'   A_m/m';' V0_g/dmh';...
                        ' P0_g/dmh'];
         for iout = 1:size(TitleString,1), fprintf(fid,'%12s', sprintf('%s',TitleString(iout,:))); end
         fprintf(fid,'\n');
      end
      % Print all the result data to text file
      for iout = 1:size(ResultsAll,1),
         fprintf(fid,'%12s%12d%s%s%s\n', sprintf('%3d  %02d %02d', ResTime(iout,1:3)), ...
                     Time(iout), sprintf('%12.4g', ResultsAll(iout,2:size(ResultsAll,2))), ...
                     sprintf('%12.4g', KLIMA(iout,:)), sprintf('%12.4g', PLANT(iout,:)));
      end
      fclose(fid);
   end

   startdatnext = States(size(States,1), :);
   SaveStartdat = 1;
   if SaveStartdat == 1,
      % Set start values of next period from actual simulation results
      startdatsave = startdatnext; startdatsave(4:5) = [0 0];
      timenext = t(size(States,1));
   
      [fid, error_message] = fopen([strtok(FileSave,'.') '.sas'], 'at');
      if fid == -1, error(sprintf('error during fopen: %s', error_message));
      else fprintf(fid,'%10d  %s\n', timenext, sprintf('%12.6g', startdatsave)); fclose(fid); end
   end
   
   
% End of function
